import type { Rule } from '@commitlint/types';
/**
 * Configuration for a single external rule to be enforced if the commit
 * is not from an ignored author.
 */
export interface ExternalRuleToEnforce {
    /**
     * The name of the NPM package that exports the rule.
     * Examples: '@commitlint/rules', 'commitlint-plugin-jira'
     */
    packageName: string;
    /**
     * The name of the rule as exported by the `rules` object of the plugin
     * specified in `packageName`.
     * Example: 'type-enum', 'header-max-length', 'jira-task-id-empty'
     */
    ruleName: string;
    /**
     * The value/options to pass to the target rule. The structure of this
     * value depends entirely on the requirements of the target rule from
     * the specified package.
     */
    value: unknown;
}
/**
 * Options for the `ignoreForAuthors` rule.
 * This rule allows for bypassing a set of specified linting rules if the
 * commit matches an author pattern, or enforcing those rules otherwise.
 */
export interface IgnoreForAuthorsRuleOptions {
    /**
     * An array of string patterns. If any of these patterns are found
     * within the raw commit message (`parsed.raw`), the `rulesToEnforce`
     * (if any) will be skipped, and this main rule will pass.
     * If omitted or empty, author pattern checking is skipped and
     * `rulesToEnforce` are always processed.
     */
    ignoreAuthorPatterns?: string[];
    /**
     * An array of external rule configurations to apply if the commit is
     * not from an ignored author (as per `ignoreAuthorPatterns`). If this
     * array is empty or not provided, and the author is not ignored, this
     * main rule will pass (as there are no specific rules to enforce).
     */
    rulesToEnforce?: ExternalRuleToEnforce[];
}
/**
 * A `commitlint` rule named `ignoreForAuthors`.
 *
 * **Behavior:**
 * 1. If `ignoreAuthorPatterns` are provided and a pattern matches the raw
 * commit message, this rule passes, and `rulesToEnforce` are bypassed.
 * 2. If no `ignoreAuthorPatterns` match (or none are provided), this rule
 * will then attempt to enforce each rule listed in `rulesToEnforce`.
 * - Each rule in `rulesToEnforce` is dynamically imported from its
 * specified `packageName`.
 * - If any enforced rule fails, this `ignoreForAuthors` rule fails with
 * the message from the underlying rule.
 * - If all enforced rules pass (or if `rulesToEnforce` is empty/undefined),
 * this rule passes.
 *
 * This rule is asynchronous due to its use of dynamic `import()`.
 *
 * @param parsed - The parsed commit object, typed as `Commit` from
 * `conventional-commits-parser`. It's expected that `commitlint` provides
 * this object augmented with a `raw: string` property at runtime.
 * @param when - Rule applicability, typically 'always' or 'never'.
 * @param value - Options for this rule, conforming to
 * `IgnoreForAuthorsRuleOptions`.
 * @returns A `Promise<RuleOutcome>`.
 */
export declare const ignoreForAuthors: Rule<IgnoreForAuthorsRuleOptions>;
