/**
 * A `commitlint` rule named `ignoreForAuthors`.
 *
 * **Behavior:**
 * 1. If `ignoreAuthorPatterns` are provided and a pattern matches the raw
 * commit message, this rule passes, and `rulesToEnforce` are bypassed.
 * 2. If no `ignoreAuthorPatterns` match (or none are provided), this rule
 * will then attempt to enforce each rule listed in `rulesToEnforce`.
 * - Each rule in `rulesToEnforce` is dynamically imported from its
 * specified `packageName`.
 * - If any enforced rule fails, this `ignoreForAuthors` rule fails with
 * the message from the underlying rule.
 * - If all enforced rules pass (or if `rulesToEnforce` is empty/undefined),
 * this rule passes.
 *
 * This rule is asynchronous due to its use of dynamic `import()`.
 *
 * @param parsed - The parsed commit object, typed as `Commit` from
 * `conventional-commits-parser`. It's expected that `commitlint` provides
 * this object augmented with a `raw: string` property at runtime.
 * @param when - Rule applicability, typically 'always' or 'never'.
 * @param value - Options for this rule, conforming to
 * `IgnoreForAuthorsRuleOptions`.
 * @returns A `Promise<RuleOutcome>`.
 */
export const ignoreForAuthors = async (parsed, when = 'always', value) => {
    if (when === 'never') {
        return [true, ''];
    }
    const options = value || {};
    const { ignoreAuthorPatterns, rulesToEnforce } = options;
    const rawCommitMessage = parsed.raw;
    if (typeof rawCommitMessage !== 'string') {
        return [
            false,
            `[@mridang/commitlint-plugin-conditionals ignoreForAuthors] Raw commit message is not available or is invalid.`,
        ];
    }
    if (ignoreAuthorPatterns && Array.isArray(ignoreAuthorPatterns)) {
        if (!ignoreAuthorPatterns.every((pattern) => typeof pattern === 'string')) {
            return [
                false,
                `[@mridang/commitlint-plugin-conditionals ignoreForAuthors] Configuration error: 'ignoreAuthorPatterns' must be an array of strings.`,
            ];
        }
        const matchedIgnoredPattern = ignoreAuthorPatterns.find((pattern) => rawCommitMessage.includes(pattern));
        if (matchedIgnoredPattern) {
            return [
                true,
                `[@mridang/commitlint-plugin-conditionals ignoreForAuthors] Commit by ignored author pattern ('${matchedIgnoredPattern}'). Ruleset defined in 'rulesToEnforce' bypassed.`,
            ];
        }
    }
    if (!rulesToEnforce ||
        !Array.isArray(rulesToEnforce) ||
        rulesToEnforce.length === 0) {
        return [
            true,
            '[@mridang/commitlint-plugin-conditionals ignoreForAuthors] No rules configured in "rulesToEnforce" for this author, or author was ignored. Rule passes.',
        ];
    }
    for (const ruleConfig of rulesToEnforce) {
        if (!ruleConfig ||
            typeof ruleConfig.packageName !== 'string' ||
            typeof ruleConfig.ruleName !== 'string') {
            return [
                false,
                `[@mridang/commitlint-plugin-conditionals ignoreForAuthors] Invalid rule configuration in 'rulesToEnforce': 'packageName' and 'ruleName' must be strings.`,
            ];
        }
        const { packageName, ruleName, value: ruleValue } = ruleConfig;
        let ruleFunction;
        try {
            // eslint-disable-next-line no-unsanitized/method
            const targetPluginModule = await import(packageName);
            const rulesObject = targetPluginModule.default?.rules || targetPluginModule.rules;
            if (rulesObject && typeof rulesObject === 'object') {
                ruleFunction = rulesObject[ruleName];
            }
        }
        catch (error) {
            const errorMessage = error instanceof Error ? error.message : String(error);
            return [
                false,
                `[@mridang/commitlint-plugin-conditionals ignoreForAuthors] Error loading plugin package '${packageName}': ${errorMessage}`,
            ];
        }
        if (typeof ruleFunction !== 'function') {
            return [
                false,
                `[@mridang/commitlint-plugin-conditionals ignoreForAuthors] Rule '${ruleName}' not found or not a function in package '${packageName}'.`,
            ];
        }
        const outcome = await Promise.resolve(ruleFunction(parsed, 'always', ruleValue));
        const [isValid, message] = outcome;
        if (!isValid) {
            return [
                false,
                `[@mridang/commitlint-plugin-conditionals via ${packageName}/${ruleName}] ${message}`,
            ];
        }
    }
    return [
        true,
        '[@mridang/commitlint-plugin-conditionals ignoreForAuthors] All configured "rulesToEnforce" passed for this author.',
    ];
};
//# sourceMappingURL=ignore-for-authors.js.map